# This code is copyrighted by Stephan Richter in August, 2000.
#
# written by Stephan Richter
# Last updated: March 18, 2001

from Products.DBObjects import DBObjectFolder, FolderishDBObject, DBTable, DBAttribute 
from Products.DBObjects import DBObjectSQLContainerSupport
from Globals            import HTMLFile

manage_addClassFolderForm = HTMLFile('dtml/classFolder_manageAddForm', globals())

def manage_addClassFolder(self, id, title='', REQUEST=None, showManage=1):
    '''add a new Class Folder'''

    id    = str(id)
    pf = ClassFolder(self, id, title)
    self._setObject(id, pf)

    self._getOb(id)._afterAdd(self, REQUEST)

    if REQUEST and showManage:
        return self.manage_main(self, REQUEST)


class Class(FolderishDBObject):
    '''contains/manages data associated with a Class'''

    meta_type = 'Class'
    icon = 'misc_/ZQuest/class'

    useUndo   = 0

    dbTable = ( DBTable('Class', [],
                        {'instructorid' : DBAttribute('instructorid', 'int', 'int'),
                         'name'         : DBAttribute('name',         'varchar(50)'),
                         'coursenumber' : DBAttribute('coursenumber', 'varchar(20)'),
                         'description'  : DBAttribute('description',  'text')})
                         + FolderishDBObject.dbTable )

    allowedSubObjects = ('ClassStudentFolder', 'ClassProblemSetFolder', 'HomepageFolder')

    titleDBRepresentation = 'name'

    manageRoles      = ('ManagerOfClass%(classid)s',) 


    def getClassId(self):
        '''getClassId(self) --> get the id of this class; return int'''
        print self.id
        return self.id



_qry_getClassesOfStudent = '''
  SELECT Class.*
  FROM Class, ClassStudent
  WHERE
    ClassStudent.personid = <dtml-sqlvar personid type="int"> AND
    ClassStudent.containerid = Class.id;
'''

_qry_getClassesOfInstructor = '''
  SELECT Class.*
  FROM Class
  WHERE
    Class.instructorid = <dtml-sqlvar personid type="int">;
'''

_qry_getProblemSetsOfStudent = '''
  SELECT ClassProblemSet.*, ProblemSet.name, ProblemSet.introduction,
          ProblemSet.ispublished, ProblemSet.problemsGiven, ProblemSet.maxscore
  FROM Class, ClassStudent, ClassProblemSet, ProblemSet
  WHERE
    ClassStudent.personid        =  <dtml-sqlvar personid type="int"> AND
    ClassStudent.containerid     =  Class.id AND
    Class.id                     =  ClassProblemSet.containerid AND
    ClassProblemSet.problemsetid =  ProblemSet.id AND
    ProblemSet.ispublished       =  true AND
    startdatetime                <= 'now'::abstime AND
    duedatetime                  >= 'now'::abstime;
'''


class ClassFolder(DBObjectFolder, DBObjectSQLContainerSupport):
    '''ClassFolder class - contains all Classs.'''

    connectionID         = 'dbConnection'
    isSQLMethodContainer = 1
    isRoleSupportRoot    = 1

    meta_type = 'ClassFolder'  
    icon = 'misc_/ZQuest/classFolder'

    useUndo   = 0
    
    manage_options = ( DBObjectFolder.manage_options
                       + DBObjectSQLContainerSupport.manage_options )

    allowedSubObjects = ('Class',)

    localManageRoles = ('Instructor',)
    manageRoles      = ('ManagerOfClasses',) 
    viewRoles        = ('Student', 'Instructor')

    sqlMethods = ( ('qry_getClassesOfStudent', 'personid=0', _qry_getClassesOfStudent),
                   ('qry_getClassesOfInstructor', 'personid=0', _qry_getClassesOfInstructor),
                   ('qry_getProblemSetsOfStudent', 'personid=0', _qry_getProblemSetsOfStudent) )


    def getClassesOfStudent(self, pid):
        '''getClassesOfStudent(self, pid) --> get all the classes for a student; return dbrows;'''

        container = self._getSQLMethodContainer(self)
        return container.qry_getClassesOfStudent(personid=pid)


    def getClassesOfInstructor(self, pid):
        '''getClassesOfInstructor(self, pid) --> get all the classes for an instructor; return dbrows;'''

        container = self._getSQLMethodContainer(self)
        return container.qry_getClassesOfInstructor(personid=pid)


    def getProblemSetsOfStudent(self, pid):
        '''getProblemSetsOfStudent(self, pid) --> get all the problem sets for a student; return dbrows;'''

        container = self._getSQLMethodContainer(self)
        return container.qry_getProblemSetsOfStudent(personid=pid)






