# This code is copyrighted by Stephan Richter in August, 2000.
#
# written by Stephan Richter
# Last updated: March 18, 2001

from Products.DBObjects import DBObjectFolder, DBObject, DBTable, DBAttribute 


class ClassProblem(DBObject):
    '''ClassProblem class -  contains/manages data associated with a ClassProblem'''

    meta_type = 'ClassProblem'
    icon = 'misc_/ZQuest/problem'

    useUndo   = 0

    dbTable = ( DBTable('ClassProblem', [],
                        {'problemid' : DBAttribute('problemid',  'int', 'int'),
                         'attempts'  : DBAttribute('attempts',   'int', 'int'),
                         'location'  : DBAttribute('location',   'int')})
                + DBObject.dbTable )


    def setLocation(self, relativeLocation):
        '''setLocation(self, relLocation) --> set a new location for this ClassProblem; return None'''

        # grep the current location of this Class Problem
        currentLocation = int(self.getAttributes()[0].location)

        # Now interpret the the relativeLocation string; very cool; 
        if relativeLocation[0] == '+':
            newLocation = currentLocation + int(relativeLocation[1:])
        elif relativeLocation[0] == '-':
            newLocation = currentLocation - int(relativeLocation[1:])
        elif relativeLocation == 'top':
            newLocation = 1
        elif relativeLocation == 'bottom':
            newLocation = self.getLastLocation()
        else:
            return None

        # Alright, let's get the SQL container and then change the locations; the cool stuff
        # is done in a PL/PgSQL function
        container = self._getSQLMethodContainer(self)
        container = self.sql_changeClassProblemLocation(id=self.id, location=newLocation)


    def getProblemAttributes(self, problemid):    
        '''getProblemAttributes(self, problemid) --> get a Problems\' attributes; return dbrows'''
        # Well, since ClassProblem only references the real Problem, I just supply a simple
        # method that will get us the Problem attributes
        container = self._getSQLMethodContainer(self)
        return container.qry_getProblem(id=problemid)



def sortByLocation(x, y):
    '''sortByLocation(x, y) --> defines the parameter the list of objects is sorted by; return boolean'''
    # just simply make sure the locations of the objects are compared. I LOVE PYTHON!
    return cmp(x['location'], y['location']);



_sql_changeClassProblemLocation = '''SELECT setnewlocation(&dtml-id;, &dtml-location;);'''

_qry_getLastLocation = '''
SELECT max(location)
  FROM ClassProblem
  WHERE containerid = <dtml-sqltest classproblemsetid type="int">'''


_sql_initClassProblemFunctions = """
<dtml-if dropFirst>
  DROP FUNCTION getnextlocation(INT8)<dtml-var sql_delimiter>
  DROP FUNCTION setnewlocation(INT8, INT)<dtml-var sql_delimiter>
  
  DROP FUNCTION defineLocation()<dtml-var sql_delimiter>
  DROP TRIGGER defineLocation ON classproblem<dtml-var sql_delimiter>
  
  DROP FUNCTION moveclassproblemtobottom()<dtml-var sql_delimiter>
  DROP TRIGGER deleteclassproblem ON classproblem<dtml-var sql_delimiter>
</dtml-if>

CREATE FUNCTION getnextlocation (INT8) 
RETURNS INT AS '
  DECLARE 
    setid ALIAS FOR $1;
    loc  INT;
  BEGIN
    SELECT MAX(location) INTO loc 
      FROM ClassProblem 
      WHERE containerid = setid;
    IF loc = 0 THEN
      RETURN 1;
    ELSE
      RETURN loc+1;
    END IF;
  END;
' LANGUAGE 'plpgsql'
<dtml-var sql_delimiter>


CREATE FUNCTION defineLocation() RETURNS OPAQUE AS '
  BEGIN
    NEW.location := getnextlocation(NEW.containerid);
    RETURN NEW;
  END;
' LANGUAGE 'plpgsql'
<dtml-var sql_delimiter>


CREATE TRIGGER defineLocation BEFORE INSERT ON classproblem
  FOR EACH ROW EXECUTE PROCEDURE defineLocation()
<dtml-var sql_delimiter>


CREATE FUNCTION moveclassproblemtobottom() RETURNS OPAQUE AS '
  DECLARE
    pid INT8;
    npid INT8;
  BEGIN
    SELECT id INTO pid 
      FROM classproblem 
      WHERE location = OLD.location+1
        AND containerid = OLD.containerid;
    npid := setnewlocation(pid, OLD.location);
    RETURN OLD;
  END;
' LANGUAGE 'plpgsql'
<dtml-var sql_delimiter>


CREATE TRIGGER deleteclassproblem AFTER DELETE ON classproblem
  FOR EACH ROW EXECUTE PROCEDURE moveclassproblemtobottom()
<dtml-var sql_delimiter>


CREATE FUNCTION setnewlocation(INT8, INT) RETURNS INT AS '
  DECLARE
    pid         ALIAS FOR $1;
    newloc      ALIAS FOR $2;
    sid         INT8;
    oldloc      INT;
    counter     INT;
    prevprobloc INT8;
    masterprobloc INT8;
  BEGIN
   SELECT location INTO masterprobloc 
   FROM classproblem 
   WHERE problemid IN (
     SELECT problem.id 
     FROM problem*, classproblem 
     WHERE 
       problem.previousproblemid = classproblem.problemid AND 
       classproblem.id = pid
   );
 
   SELECT location INTO prevprobloc 
      FROM classproblem 
      WHERE problemid IN (
        SELECT previousproblemid 
        FROM problem*, classproblem 
        WHERE 
          problem.id = classproblem.problemid AND 
          classproblem.id = pid
       )
       LIMIT 1;

    IF ((masterprobloc = NULL) OR (masterprobloc > newloc)) AND 
       ((prevprobloc = NULL) OR (prevprobloc < newloc)) THEN

      SELECT location INTO oldloc FROM ClassProblem WHERE id = pid;
      SELECT containerid INTO sid FROM ClassProblem WHERE id = pid;

      IF oldloc > newloc THEN
        UPDATE ClassProblem SET location = location + 1 
          WHERE location <  oldloc AND 
                location >= newloc  AND 
                containerid = sid;
      ELSE
        UPDATE ClassProblem SET location = location - 1 
          WHERE location > oldloc AND 
                location <= newloc AND 
                containerid = sid;
      END IF;
      UPDATE ClassProblem SET location = newloc WHERE id = pid;
    END IF;
    RETURN oldloc;
  END;
' LANGUAGE 'plpgsql'
<dtml-var sql_delimiter>
"""

_sql_initLanguage = """
<dtml-if dropFirst>
  DROP PROCEDURAL LANGUAGE 'plpgsql';
  DROP FUNCTION plpgsql_call_handler();
</dtml-if>

CREATE FUNCTION plpgsql_call_handler () RETURNS opaque
  AS '/usr/lib/pgsql/plpgsql.so'
  LANGUAGE 'C';

CREATE PROCEDURAL LANGUAGE 'plpgsql'
  HANDLER plpgsql_call_handler
  LANCOMPILER 'PL/PgSQL';
"""


class ClassProblemFolder(DBObjectFolder):
    '''ClassProblemFolder class - contains all ClassProblems for one class.'''

    meta_type = 'ClassProblemFolder'  
    icon = 'misc_/ZQuest/problemFolder'

    useUndo   = 0
    
    allowedSubObjects = ('ClassProblem',)

    sqlMethods = ( ('sql_changeClassProblemLocation', 'id location',       _sql_changeClassProblemLocation),
                   ('qry_getLastLocation',            'classproblemsetid', _qry_getLastLocation),
                   ('sql_initClassProblemFunctions',  'dropFirst:int=1',   _sql_initClassProblemFunctions),
                   ('sql_initLanguage',               'dropFirst:int=1',   _sql_initLanguage) )


    def getProblemsOrderedByLocation(self):
        '''getProblemsOrderedByLocation(self) --> get all Problems in Class Problems ordered by
           Locacation; return list of dbrows'''

        container = self._getSQLMethodContainer(self)
        dbrows = container.qry_getAllSubObjects(table='ClassProblem', orderBy='location',
                                                containertype='ClassProblemSet',
                                                containerid=self.getClassProblemSetId())

        
        return dbrows
        # now map the DB rows to a list, so we can use sort
        result = map(lambda x: x, dbrows)
        # sort the entries by location; how sort is supposed to do that is defined in sortByLocation
        result.sort(sortByLocation)

        return result
    

    def getLastLocation(self):
        '''getLastLocation(self) --> return the last location of this class problem set; return int'''
        # We need the Class Problem Set Id to run the query, so get it
        classProblemSetId = self.getParentNode().id
        # get the SQL container and then execute the simple query that gets the last location
        container = self._getSQLMethodContainer(self)
        return int(container.qry_getLastLocation(classproblemsetid=classProblemSetId)[0][0])
    

    def getAllProblems(self):
        '''manage_addAProblems(self) --> return all problems of the problem set; return None'''

        # this should be supported by another object. Fix me.

        classproblemsetid= self.getClassProblemSetId()
        problemsetid = self.ClassProblemSets[classproblemsetid].getAttributes()[0].problemsetid

        container = self._getSQLMethodContainer(self)
        return container.qry_getAllProblems(containerid=problemsetid)

