# This code is copyrighted by Stephan Richter in August, 2000.
#
# written by Stephan Richter
# Last updated: March 18, 2001

from Products.DBObjects import DBObjectFolder, FolderishDBObject, DBTable, DBAttribute 

import random, time


class ClassProblemSet(FolderishDBObject):
    '''ClassProblemSet class -  contains/manages data associated with a ClassProblemSet'''

    meta_type = 'ClassProblemSet'
    icon = 'misc_/ZQuest/classProblemSet'

    useUndo   = 0

    dbTable = ( DBTable('ClassProblemSet', [],
                      {'problemsetid'  : DBAttribute('problemsetid',  'int', 'int'),
                       'repititions'   : DBAttribute('repititions',   'int', 'int'),
                       'maxtime'       : DBAttribute('maxtime',       'interval'),
                       'startdatetime' : DBAttribute('startdatetime', 'datetime'),
                       'duedatetime'   : DBAttribute('duedatetime',   'datetime')})
                + FolderishDBObject.dbTable )

    allowedSubObjects = ('ClassProblemFolder',)


    def getClassProblemSetId(self):
        '''getClassProblemSetId(self) --> get the id of this class; return int'''
        return self.id


    def getProblemSetAttributes(self):    
        '''getProblemSetAttributes(self) --> get the Problem Set attributes; return dbrow'''

        # As usual get the SQL Method container
        container = self.ProblemSets
        # Let's get the Problem Set Id first
        problemsetid = self.getAttributes()[0].problemsetid
        # Now get the problem set attributes
        return container.qry_getProblemSet(id=problemsetid)

    def getClassProblemSetProblems(self, setId):    
        '''getClassProblemSetProblems(self, setId) --> get all Problems of this class problem set;
           return dbrows'''

        container = self._getSQLMethodContainer(self)
        return container.qry_getProblemsOfClassProblemSet(classproblemsetid=setId)


    def _generateUniqueRandomSequence(self, numOfElements, largestValue):
        '''_generateUniqueRandomSequence(self, numOfElements, largestValue) --> generate
           a sequence (list) of unique intergers between 0 and largestValue; return list;'''

        random.seed(time.time())
        numbers = []
        while len(numbers) < numOfElements and largestValue >= 0:
            number = random.randint(0, largestValue)
            if numOfElements > largestValue+1 or not number in numbers:
                numbers.append(number)

        return numbers
        

    def getClassProblemsForSession(self):
        '''getClassProblemsForSession(self) --> returns a list of class problems;
           return list of rows'''

        if len(self.ClassProblems.objectIds('ClassProblem')) != 0:
            selectedProblems=self.ClassProblems.getProblemsOrderedByLocation()
            #selectedProblems = map(lambda x: x.problemid, problems)

        else:
            problemSet = self.ProblemSets[str(self.getAttributes()[0].problemsetid)]
            problemSetAttributes = problemSet.getAttributes()[0]

            selectedProblems = []
            problems = map(lambda x, c=PseudoClassProblem: c(x.id), problemSet.Problems.getAllSubObjects())

            for position in self._generateUniqueRandomSequence(problemSetAttributes.problemsgiven,
                                                               len(problems)-1):
                selectedProblems.append(problems[position])

        return selectedProblems


class PseudoClassProblem:
    ''' '''
    
    def __init__(self, problemid, attempts=1):
        self.problemid = problemid
        self.attempts = attempts
                


_qry_getProblemsOfClassProblemSet = '''
SELECT Problem.*
FROM ClassProblemSet, Problem* as prob
WHERE
  ClassProblemSet.id = <dtml-sqlvar classproblemsetid type="int"> AND
  prob.problemsetid = ClassProblemSet.problemsetid
'''


class ClassProblemSetFolder(DBObjectFolder):
    '''ClassProblemSetFolder class - contains all ClassProblemSets for one class.'''

    meta_type = 'ClassProblemSetFolder'  
    icon = 'misc_/ZQuest/classProblemSetFolder'

    useUndo   = 0
    
    allowedSubObjects = ('ClassProblemSet',)

    sqlMethods = ( ('qry_getProblemsOfClassProblemSet', 'classproblemsetid=0', _qry_getProblemsOfClassProblemSet), )
