# This code is copyrighted by Stephan Richter in August, 2000.
#
# written by Stephan Richter
# Last updated: August 28, 2000

from Products.DBObjects import DBObjectFolder, DBObject, DBTable, DBAttribute 

import Tools


class MatchPair(DBObject):
    '''MatchPair class -  contains/manages data associated with a MatchPair'''

    meta_type = 'MatchPair'
    icon = 'misc_/ZQuest/matchPair'

    useUndo   = 0

    dbTable = ( DBTable('MatchPair', [],
                        {'key'       : DBAttribute('key',       'text'),
                         'value'     : DBAttribute('value',     'text')})
                + DBObject.dbTable )   

    titleDBRepresentation = 'key'



class MatchPairFolder(DBObjectFolder):
    '''MatchPairFolder class - contains all MatchPairs for one MatchProblem.'''

    meta_type = 'MatchPairFolder'  
    icon = 'misc_/ZQuest/matchPairFolder'

    useUndo   = 0
    
    allowedSubObjects = ('MatchPair',)


    def getKeyValueDictionary(self):
        '''getKeyValueDictionary(self) --> return a dictionary, that maps a key and a value;
           return dictionary of key: value'''

        dict = {}

        # let's get the match pairs from the DB
        container = self._getSQLMethodContainer(self)
        attributes = container.qry_getAllMatchPairs(containerid=self.getProblemId())

        for attribute in attributes:
            dict[attribute.key] = attribute.value

        return dict


    def getKeys(self):
        '''getKeys(self) --> get all keys from the match pairs; return list'''

        return self.getKeyValueDictionary().keys()
        

    def getValues(self):
        '''getValues(self) --> get all values from the match pairs; return list'''

        return self.getKeyValueDictionary().values()


    def getKeysInRandomOrder(self):
        '''getKeysInRandomOrder(self) --> return the keys found with getKeys() in a random order,
           so that the question becomes more interesting; return list'''

        return Tools.randomizeList(self.getKeys())


    def getValuesInRandomOrder(self):
        '''getValuesInRandomOrder(self) --> return the values found with getValues() in a random order,
           so that the question becomes more interesting; return list'''

        return Tools.randomizeList(self.getValues())


    def getNumberOfCorrectMatches(self, answerMatches):
        '''getNumberOfCorrectMatches(self, dict) --> check the answered matches with the correct ones;
           the method returns the number of correct matches; return int'''

        correctMatches = self.getKeyValueDictionary()
        correctAnswers = 0

        for key in answerMatches.keys():
            if answerMatches[key] == correctMatches[key]:
                correctAnswers = correctAnswers + 1

        return correctAnswers


    def getResultsFromRequest(self, REQUEST):
        '''getResultsFromRequest(self, REQUEST) --> based on the variables in the REQUEST variable,
           generate the a result; return int'''

        numberOfMatchPairs = len(self.objectIds())
        answerMatches = {}
        for number in range(0, numberOfMatchPairs):
            answerMatches[REQUEST['key_%i' %number]] = REQUEST['match_%i' %number]

        return self.getNumberOfCorrectMatches(answerMatches)
