# This code is copyrighted by Stephan Richter in August, 2000.
#
# written by Stephan Richter
# Last updated: March 18, 2001

from DocumentTemplate   import DT_HTML
from Problem            import Problem
from Products.DBObjects import DBTable, DBAttribute 
from Tools              import randomizeList

class MultipleChoiceProblem(Problem):
    '''MultipleChoiceProblem class -  contains/manages data associated with a Problem'''

    meta_type = 'MultipleChoiceProblem'

    dbTable = DBTable('MultipleChoiceProblem', ['Problem'], 
                      { 'isorderrandom'           : DBAttribute('isorderrandom',           'boolean'),
                        'choicesdisplaydirection' : DBAttribute('choicesdisplaydirection', 'int', 'int')})

    allowedSubObjects = ('ChoiceFolder', 'DisplayObjectFolder', 'GivenVariableFolder')


    def getChoices(self):
        '''getChoices(self) --> get the choices for the problem; return list'''
        container = self._getSQLMethodContainer(self)
        choices = []
        for choice in container.qry_getChoicesOfProblem(problemid=self.id):
            choices.append(choice)

        if self.getAttributes()[0].isorderrandom == 't':
            choices = randomizeList(choices)

        return choices


    def evalValue(self, value, REQUEST):
        ''' '''
        html = DT_HTML.HTML(value)
        return html(client=self, mapping=REQUEST)


    def isAnswerCorrect(self, answerChoiceId):
        '''isAnswerCorrect(self, answerChoiceId) --> checks whether the answer provided by the user was
           correct; return boolean'''

        # check in the database whether this choice is the correct one
        container = self._getSQLMethodContainer(self)
        result = self.qry_checkMultipleChoiceProblemAnswer(id=answerChoiceId)

        # return the result
        return int(result[0].iscorrect)


    def getScoreForAnswer(self, REQUEST):
        '''getScoreForAnswer(self, REQUEST) --> returns the score achieved and max score of the problem;
           return (score, maxscore)'''

        maxscore = float(self.getAttributes()[0].maxscore)
        score = self.isAnswerCorrect(REQUEST['choice'])*maxscore
        
        return (score, maxscore)

