# This code is copyrighted by Stephan Richter in August, 2000.
#
# written by Stephan Richter
# Last updated: March 18, 2001

from Problem            import Problem
from Products.DBObjects import DBTable, DBAttribute 


class NumericProblem(Problem):
    '''NumericProblem class -  contains/manages data associated with a Problem'''

    meta_type = 'NumericProblem'

    dbTable = DBTable('NumericProblem', ['Problem'])

    allowedSubObjects = ('GivenVariableFolder', 'SearchedVariableFolder', 'DisplayObjectFolder' )
 

    def evalSearchedVariables(self, givenVariablesDict):
        '''evalSearchedVariable(self, givenVariableDict) --> return a dictionary of all the searched variables
        and their values; return dictionary of varname: [min, max]'''

        # define the dictionary that will contain the results
        searchedVariablesDict = {}

        # let's generate a new virtual environment
        environ = rexec.RExec()
        environ.r_exec('from math import *')

        # now add all the declarations to the environment
        for variable in givenVariablesDict.keys():
            defString = variable + '=' + str(givenVariablesDict[variable])
            environ.r_exec(defString)

        # for every searched variable, do the following
        for variable in self.SearchedVariables.objectValues():
            
            # get the necessary attributes from the DB
            attributes = variable.getAttributes()[0]
            # execute the code that will evaluate 
            environ.r_exec(string.replace(attributes.evalfunction, '\015\012', '\n'))

            sigFig = abs(math.log(attributes.errortolerancefactor)/math.log(10))+1
            value = Tools.roundNumberToSigFig(environ.r_eval(attributes.variablename), sigFig)
            errFactor = attributes.errortolerancefactor
            
            # save the evaluated variable
            lowerLimit = value-abs(value)*errFactor
            upperLimit = value+abs(value)*errFactor
            searchedVariablesDict[attributes.variablename] = [lowerLimit, upperLimit]

        # return the searched variables dictionary
        return searchedVariablesDict


    def getPreviousProblemVariables(self, REQUEST):
        '''getPreviousProblemVariables(self) --> return dictionary'''

        previousProblemId = self.getAttributes()[0].previousproblemid
        if int(previousProblemId) == 0:
            return {}
        else:
            problemSet = self.getParentNode()
            return problemSet[str(previousProblemId)].getAllVariableValues(REQUEST)


    def compareAnswers(self, REQUEST):
        '''compareAnswers(self, REQUEST) --> '''

        # setup the two name space dictionaries
        givenVariablesDict = self.getPreviousProblemVariables(REQUEST)
        searchedVariablesDict = {}

        # The given variables are in REQUEST.other; Note that they are not passed in
        # the form for security reasons
        for variable in self.GivenVariables.objectValues():
            if REQUEST.has_key(variable.title):
                givenVariablesDict[variable.title] = float(REQUEST[variable.title])

        # The searched variables are provided by the user; that's why look in the form
        # Note: REQUEST.other contains also the results (the correct ones); it gets them
        # from the session variables
        for variable in self.SearchedVariables.objectValues():
            if REQUEST.has_key(variable.title):
                searchedVariablesDict[variable.title] = float(REQUEST.form[variable.title])

        trueSearchedVariablesDict = self.evalSearchedVariables(givenVariablesDict)

        correct = 0
        total = 0
        for variable in trueSearchedVariablesDict.keys():

            if (searchedVariablesDict[variable] > trueSearchedVariablesDict[variable][0] and
                searchedVariablesDict[variable] < trueSearchedVariablesDict[variable][1]):
                correct = correct + 1
            total = total + 1

        return (correct, total)        
        

    def getScoreForAnswer(self, REQUEST):
        '''getScoreForAnswer(self, REQUEST) --> returns the score achieved and max score of the problem;
           return (score, maxscore)'''

        maxscore = float(self.getAttributes()[0].maxscore)
        results = self.compareAnswers(REQUEST)

        score = (results[0]/results[1])*maxscore
        
        return (score, maxscore)


    def _setVariablesInREQUEST(self, REQUEST):
        '''_setVariablesInREQUEST(self, REQUEST) --> set the given and searched variables in REQUEST;
            return None'''

        givenVariablesDict = self.getPreviousProblemVariables(REQUEST)

        for variable in givenVariablesDict.keys():
            REQUEST.set(variable, givenVariablesDict[variable])

        for variable in self.GivenVariables.objectValues():
            givenVariablesDict[variable.title] = float(REQUEST[variable.title])
            REQUEST.set(variable.title, float(REQUEST[variable.title]))

        searchedVariablesDict = self.evalSearchedVariables(givenVariablesDict)
        for variable in searchedVariablesDict.keys():
            REQUEST.set(variable, float((searchedVariablesDict[variable][0]+searchedVariablesDict[variable][1])/2))
        
        return REQUEST


    def getExplanation(self, REQUEST):
        '''getExplanation(self, REQUEST) --> return the explanation parsed for dtml syntax; return string'''

        REQUEST = self._setVariablesInREQUEST(REQUEST)

        description = self.getAttributes()[0].description
        html = DT_HTML.HTML(description)
        
        return html(client=self, mapping=REQUEST.other)


    def getExplanation(self, REQUEST):
        '''getExplanation(self, REQUEST) --> return the explanation parsed for dtml syntax; return string'''

        REQUEST = self._setVariablesInREQUEST(REQUEST)

        explanation = self.getAttributes()[0].explanation
        html = DT_HTML.HTML(explanation)
        
        return html(client=self, mapping=REQUEST.other)


    def getAllVariableValues(self, REQUEST):
        '''getAllVariableValues(self, REQUEST) --> Returns a dictionary of variables and their value from the
           REQUEST. This method is used to get all the varibales to be saved for a session; return dict'''

        variables = self.getPreviousProblemVariables(REQUEST)

        for variable in self.GivenVariables.objectValues():
            if REQUEST.has_key(variable.title):
                variables[variable.title] = float(REQUEST[variable.title])

        searchedVariables = self.evalSearchedVariables(variables)

        for variable in searchedVariables.keys():
            variables[variable] = float((searchedVariables[variable][0]+searchedVariables[variable][1])/2)

        return variables
