# This code is copyrighted by Stephan Richter in August, 2000.
#
# written by Stephan Richter
# Last updated: August 28, 2000

from Products.DBObjects         import DBObjectFolder, FolderishDBObject, DBTable, DBAttribute 
from Products.DBObjects.Support import UserSupport
import Tools


class Person(FolderishDBObject, UserSupport):
    '''contains/manages data associated with a Person'''

    meta_type = 'Person'
    icon      = 'misc_/ZQuest/person'

    useUndo   = 0

    dbTable = ( DBTable('Person', [],
                      {'username' : DBAttribute('username', 'varchar(100)'),
                       'password' : DBAttribute('password', 'varchar(100)'),
                       'roles'    : DBAttribute('roles',    'varchar(255)'),
                       'domains'  : DBAttribute('domains',  'varchar(100)'),
                       'fname'    : DBAttribute('fname',    'varchar(30)'),
                       'lname'    : DBAttribute('lname',    'varchar(30)'),
                       'address1' : DBAttribute('address1', 'varchar(50)'),
                       'address2' : DBAttribute('address2', 'varchar(50)'),
                       'city'     : DBAttribute('city',     'varchar(30)'),
                       'state'    : DBAttribute('state',    'varchar(2)'),
                       'zip'      : DBAttribute('zip',      'varchar(10)'),
                       'phone'    : DBAttribute('phone',    'varchar(12)'),
                       'fax'      : DBAttribute('fax',      'varchar(12)'),
                       'email'    : DBAttribute('email',    'varchar(200)'),
                       'url'      : DBAttribute('url',      'varchar(200)')})
                      + UserSupport.dbTable
                      + FolderishDBObject.dbTable )

    titleDBRepresentation = 'email'
    allowedSubObjects     = ('ScoreFolder',)

    manageRoles = ('Person%(personid)s',)

    __ac_permissions__ = ( UserSupport.__ac_permissions__ +
                           FolderishDBObject.__ac_permissions__ )

    manage_options = ( FolderishDBObject.manage_options +
                       UserSupport.manage_options )
    

    def getSpecialRoles(self):
        '''Get roles that are dependent on this system'''
        roles = map(lambda x: 'ManagerOfSession'+str(x['id']), self.Sessions.getAllSessionsOfPerson(self.id))
        roles = roles + map(lambda x: 'ManagerOfClass'+str(x['id']), self.Classes.getClassesOfInstructor(self.id))
        return roles
    

    def getPersonId(self):
        '''getPersonId(self) --> return the person\'s id; return id'''
        return self.id



_qry_searchAllPersons = '''
<dtml-if "fname == '' and lname == '' and email == ''">
    SELECT id, fname, lname, email
    FROM Person
    WHERE id=0
<dtml-else>
  SELECT id, fname, lname, email
  FROM Person
  WHERE
  
    <dtml-if fname>
      fname ~* '^&dtml-fname;' AND
    </dtml-if>

    <dtml-if lname>
      lname ~* '^&dtml-lname;' AND
    </dtml-if>

    <dtml-if email>
      email ~* '^&dtml-email;' AND
    </dtml-if>
  
    1=1
</dtml-if>
'''



class PersonFolder(DBObjectFolder):
    '''contains all Persons for one project.'''

    meta_type = 'PersonFolder'  
    icon      = 'misc_/ZQuest/personFolder'

    useUndo   = 0
    
    allowedSubObjects = ('Person',)

    manageRoles = ('ManagePersons',)

    sqlMethods = ( ('qry_searchAllPersons', 'fname="" lname="" email=""', _qry_searchAllPersons),)


    def search(self, fname='', lname='', email=''):
        '''search(self, fname='', lname='', email='') --> search for persons; return dbrows;'''

        toRegex = Tools.replaceStarForRegExStar

        container = self._getSQLMethodContainer(self)
        return container.qry_searchAllPersons(fname=toRegex(fname), lname=toRegex(lname), email=toRegex(email))
