# This code is copyrighted by iuveno AG in January, 2001.
#
# written by Stephan Richter
# Last updated: February 3, 2001

import string, base64, urllib

from Products.DBObjects          import DBObjectFolder, FolderishDBObject, DBTable, DBAttribute
from Products.DBObjects          import DBObjectSQLContainerSupport
from Products.DBObjects.Support  import TreeViewSupportObject, TreeViewSupportFolder
from Products.DBObjects.Support  import FlatTreeViewSupportObject, FlatTreeViewSupportFolder
from Products.DBObjects.Support  import UserFolderSupport
from Person                      import Person

from Globals               import MessageDialog
from OFS.DTMLMethod        import DTMLMethod
from OFS.ObjectManager     import ObjectManager
from DateTime              import DateTime


def manage_addMainPersonGroupFolder(self, title='', REQUEST=None, container=None, showManage=1):
    '''manage_addPersonGroupFolder(self, title='', REQUEST=None) --> Add a PersonGroupFolder to
       the ZODB; return None or HTML'''

    
    if not container:
        container = self.aq_parent.aq_parent.this()

    ob = MainPersonGroupFolder(container, title, REQUEST)

    try:
        container._setObject('acl_users', ob)
        container['acl_users']._afterAdd(container, REQUEST)
    except:
        return MessageDialog(
                 title   = 'Item Exists',
                 message = 'This object already contains a Person Folder',
                 action  = '%s/manage_main' % REQUEST['URL1'])
    
    container.__allow_groups__ = ob
    
    if REQUEST and showManage:
        return self.manage_main(self, REQUEST, update_menu=1)



class PersonGroup(FolderishDBObject, TreeViewSupportObject, FlatTreeViewSupportObject):
    '''PersonGroup class - contains and manages a group of persons'''

    icon      = 'misc_/ZQuest/personGroup'
    meta_type = 'PersonGroup'

    useUndo   = 0

    allowedSubObjects  = ('PersonGroupFolder', 'PersonFolder')

    dbTable = (DBTable('PersonGroup', [],
                       {'name'        : DBAttribute('name',        'varchar(100)'  ),
                        'description' : DBAttribute('description', 'text' ),
                        'roles'       : DBAttribute('roles',       'text' ),
                        })
               + FolderishDBObject.dbTable)

    titleDBRepresentation = 'name'

    manageRoles = ('ManagerOfGroup%(persongroupid)s', 'ManageGroups')
    viewRoles   = ('ViewGroups', 'ViewGroup%(persongroupid)s')        


    def getLocalGroupRoles(self):
        '''getGroupRoles(self) --> get all the roles of a group; return list'''
        roles = ['PersonOfGroup%s' %self.id]
        roles = roles + filter(None, string.split(self.getAttributes()[0].roles, ','))
        return roles


    def getAllInheritedGroupRoles(self):
        '''getAllInheritedGroupRoles(self) --> get all the roles of this and all previous groups; return list'''
        roles = self.getLocalGroupRoles()
        if hasattr(self.aq_parent, 'getAllInheritedGroupRoles'):
            roles = roles + self.aq_parent.getAllInheritedGroupRoles()
        return roles


    def getPersonGroupId(self):
        '''getPersonGroupId(self) --> get the id of the object; return string'''
        return self.id



_qry_searchForPersons = '''
SELECT *
  FROM Person
  <dtml-sqlgroup where>
    <dtml-if name>
      <dtml-sqlgroup>
        firstname  ~* <dtml-sqlvar name type="string">
      <dtml-or>
        middlename ~* <dtml-sqlvar name type="string">
      <dtml-or>
        lastname   ~* <dtml-sqlvar name type="string">
      </dtml-sqlgroup>
    </dtml-if>
  <dtml-or>
    <dtml-if phone>
      phone        ~* <dtml-sqlvar phone type="string">
    </dtml-if>
  <dtml-or>
    <dtml-if email>
      email        ~* <dtml-sqlvar email type="string">
    </dtml-if>
  </dtml-sqlgroup>
'''


class PersonGroupFolder(DBObjectFolder, ObjectManager, UserFolderSupport, TreeViewSupportFolder,
                        FlatTreeViewSupportFolder):
    '''PersonGroupFolder class - contains Person Groups'''

    meta_type = 'PersonGroupFolder'
    id        = 'acl_users'
    icon      = 'misc_/ZQuest/personGroupFolder'

    useUndo   = 0

    manageRoles = ('ManageGroups',)
    viewRoles   = ('ViewGroups',)

    allowedSubObjects = ('PersonGroup',)

    sqlMethods = ( (('qry_searchForPersons', 'name email phone', _qry_searchForPersons),)
                   + UserFolderSupport.sqlMethods )

    manage_options = DBObjectFolder.manage_options

    __ac_permissions__ = DBObjectFolder.__ac_permissions__

    authAttributes = Person.authAttributes


class MainPersonGroupFolder(PersonGroupFolder, DBObjectSQLContainerSupport):
    '''MainPersonGroupFolder class - contains Person Groups and is a DBObjects Root Object'''

    connectionID         = 'dbConnection'
    isSQLMethodContainer = 1
    isRoleSupportRoot    = 1

    meta_type = 'MainPersonGroupFolder'

    manage_options = ( PersonGroupFolder.manage_options +
                       DBObjectSQLContainerSupport.manage_options )

    __ac_permissions__ = (UserFolderSupport.__ac_permissions__ +
                          PersonGroupFolder.__ac_permissions__ +
                          (('View', ('getUser','')),))


    def __init__(self, parent, title='', request=None):
        '''__init__(self, parent, request, title=None, basedn=None) --> Initialize the object;
          return None'''

        # Okay, because we can now have PersonGroupFolders in PersonGroups themselves, we have to
        # be careful, when we generate the loginForm script
        self.loginForm = DTMLMethod(_loginForm %(request['URL0']), __name__='loginForm')
    
        for subObject in self.allowedSubObjects:
            self.getSubObjectClass(subObject).dbTable.createBasicSQLMethods(self)

        self.id     = 'acl_users'
        self.title  = title

        self.initSQLMethods(self)
        

    def _reconnect(self):
        '''_reconnect(self) --> Reconnects the database to Zope; return None'''
        conn = getattr(self, 'pdConnection')
        conn.connect(conn.connection_string)


    def getPersonById(self, personId):
        '''getPersonById(self, personId) --> gets a person by his/her id; return dbRow;'''
        container = self._getSQLMethodContainer(self)
        return container.qry_getPersonById(personid = personId)


    def getPersonObjectById(self, personId):
        ''' '''
        dbPerson = self.getPersonById(personId)
        return self.convertSQLResultToObjects(dbPerson, 'Person')[0]


    def getPersonDictionary(self):
        '''getPersonDictionary(self) --> returns a dictionary of person ids and names; return dict'''
        dict = {}
        container = self._getSQLMethodContainer(self)
        dbRows = container.qry_getAllPersonsOfAllGroups()

        for row in dbRows:
            dict[int(row.id)] = row.firstname + ' ' + row.lastname

        return dict


    def existsUserWithUsername(self, username):
        '''existsUserWithUsername(self, username) --> checks whether a username already exists;
           return boolean;'''
        container = self._getSQLMethodContainer(self)
        dbRows = container.qry_getAllPersonsOfAllGroups()
        usernames = map(lambda x: x.username, dbRows)
        if username in usernames:
            return 1
        else:
            return 0


    def searchForPersons(self, argName, argEmail, argPhone):
        '''searchForPersons(self, argName, argEmail, argPhone) --> return matched persons;
           return dbRows'''
        container = self._getSQLMethodContainer(self)
        return container.qry_searchForPersons(name=argName, email=argEmail, phone=argPhone)


_loginForm = '''
<html>
  <head>
    <title>Perspectives</title>
    <meta http-equiv="Refresh" content="0; url=index_html?next_url=%s">
  </head>
</html>
'''

