# This code is copyrighted by Stephan Richter in August, 2000.
#
# written by Stephan Richter
# Last updated: March 18, 2001

import Globals
from DocumentTemplate   import DT_HTML
from Globals            import HTMLFile
from Products.DBObjects import DBObjectFolder, FolderishDBObject, DBTable, DBAttribute 
from Tools              import randomizeList

import Tools, rexec, math, string



class Problem(FolderishDBObject):
    '''Problem class -  contains/manages data associated with a Problem'''

    meta_type = 'Problem'
    icon = 'misc_/ZQuest/problem'

    useUndo   = 0

    dbTable = ( DBTable('Problem', [],
                        {'previousproblemid' : DBAttribute('previousproblemid',  'int',       'int'),
                         'name'              : DBAttribute('name',               'varchar(50)'),
                         'description'       : DBAttribute('description',        'text'       ),
                         'difficultylevel'   : DBAttribute('difficultylevel',    'int',       'int'),
                         'maxscore'          : DBAttribute('maxscore',           'float',     'float'),
                         'isonlinegraded'    : DBAttribute('isonlinegraded',     'boolean'    ),
                         'explanation'       : DBAttribute('explanation',        'text'       ),
                         'alwaysexplanation' : DBAttribute('alwaysexplanation',  'boolean'    )} )
                        + FolderishDBObject.dbTable )

    titleDBRepresentation = 'name'
    allowedSubObjects = ('GivenVariableFolder', 'DisplayObjectFolder')


    def getProblemType(self):
        '''getProblemType(self) --> returns the meta type of the object; return string'''
        return self.meta_type


    def getProblemId(self):
        '''getProblemId(self) --> returns the id of the object; return string'''
        return self.id


    def initProblem(self, REQUEST, extraVariables={}):
        '''initProblem(self, REQUEST) --> Initialize a Problem; return some HTML if necessary; return string'''

        # define the string that will contain the form entries
        html = ''

        for variable in self.GivenVariables.objectValues():
            # create a random generated value for the variable
            sigFig = int(variable.getAttributes()[0].significantfigures)
            value = Tools.roundNumberToSigFig(variable.getRandomValue(), sigFig)
            # put the variable with its value in the REQUEST
            REQUEST.set(variable.title, value)
            # put the variable with its value in the form as a hidden field

        return html


    def getScoreForAnswer(self, REQUEST):
        '''getScoreForAnswer(self, REQUEST) --> returns the score achieved and max score of the problem;
           return (score, maxscore)'''
        return (0, 0)


    def getExplanation(self, REQUEST):
        '''getExplanation(self, REQUEST) --> return the explanation parsed for dtml syntax; return string'''

        explanation = self.getAttributes()[0].explanation
        html = DT_HTML.HTML(explanation)
        
        return html(client=self, mapping=REQUEST)


    def getDescription(self, REQUEST):
        '''getDescription(self, REQUEST) --> return the description parsed for dtml syntax; return string'''

        description = self.getAttributes()[0].description
        html = DT_HTML.HTML(description)
        
        return html(client=self, mapping=REQUEST)


    def getAllVariableValues(self, REQUEST, extraVariables={}):
        '''getAllVariableValues(self, REQUEST) --> Returns a dictionary of variables and their value from the
           REQUEST. This method is used to get all the varibales to be saved for a session; return dict'''

        return {}



_qry_getChoicesOfProblem = '''
SELECT *
FROM Choice
WHERE containerid = <dtml-sqlvar problemid type="int">
'''

_qry_checkMultipleChoiceProblemAnswer = '''
SELECT count(*) as iscorrect
FROM Choice
WHERE
    <dtml-sqltest id type="int"> AND
    iscorrectchoice = 'true'
'''

_qry_searchAllProblems = '''
<dtml-if "name == '' and description == '' and difficultylevel == ''">
  SELECT id, name, description, difficultylevel
  FROM Problem
  WHERE id=0
<dtml-else>
  SELECT id, name, description, difficultylevel
  FROM Problem*
  WHERE

      <dtml-if name>
        name ~* '^&dtml-name;' AND
      </dtml-if>

      <dtml-if description>
        description ~* '^&dtml-description;' AND
      </dtml-if>
    
      <dtml-if difficultylevel>
        difficultylevel >= &dtml-difficultylevel; AND
      </dtml-if>

      <dtml-if problemsetid>
        containerid = <dtml-sqlvar problemsetid type="int"> AND
      </dtml-if>
  
      1=1

</dtml-if>
'''

_qry_getAllProblems = '''
SELECT *
  FROM Problem*
  WHERE problemsetid = <dtml-sqlvar containerid type="int">
'''

_qry_getProblem = '''
SELECT *
  FROM Problem*
  WHERE <dtml-sqltest id type="int">
'''


class ProblemFolder(DBObjectFolder):
    '''ProblemFolder class - contains all Problems for one project.'''

    meta_type = 'ProblemFolder'  
    icon = 'misc_/ZQuest/problemFolder'

    useUndo   = 0
    
    allowedSubObjects = ('MultipleChoiceProblem', 'TrueFalseProblem', 'NumericProblem',
                         'EssayProblem', 'MatchProblem', 'Problem')

    choicesDisplayDirections = {0: 'horizontal', 1: 'vertical'}


    sqlMethods = ( ('qry_getAllProblems', 'containerid=0', _qry_getAllProblems),
                   ('qry_getProblem', 'id=0', _qry_getProblem),
                   ('qry_getChoicesOfProblem', 'problemid=0', _qry_getChoicesOfProblem),
                   ('qry_checkMultipleChoiceProblemAnswer', 'id=0', _qry_checkMultipleChoiceProblemAnswer),
                   ('qry_searchAllProblems', 'name="" description="" difficultylevel="" problemsetid=""',
                    _qry_searchAllProblems) )


    def search(self, name='', description='', difficultylevel=''):
        '''search(self, name='', description='', difficultylevel='') --> search for class problems;
           return dbrows;'''

        toRegex = Tools.replaceStarForRegExStar
        resultList = []

        container = self._getSQLMethodContainer(self)
        return container.qry_searchAllProblems(name            = toRegex(name),
                                               description     = toRegex(description),
                                               difficultylevel = toRegex(difficultylevel),
                                               problemsetid    = self.getProblemSetId())


    def isProblemReferenced(self, problemId):
        '''isProblemReferenced(self, problemId) --> check whether the problem is referenced in another
           problem as a previous problem; return boolean'''

        container = self._getSQLMethodContainer(self)
        rows = self.getAllSubObjectsWithQuery('qry_getAllProblems')

        for row in rows:
            if int(row.previousproblemid) == int(problemId):
                return 1

        return 0
        


Globals.default__class_init__(ProblemFolder)






