# This code is copyrighted by Stephan Richter in August, 2000.
#
# written by Stephan Richter
# Last updated: March 18, 2001

from Globals            import HTMLFile
from DocumentTemplate   import DT_HTML
from Products.DBObjects import DBObjectFolder, FolderishDBObject, DBTable, DBAttribute 
from Products.DBObjects import DBObjectSQLContainerSupport
from Problem            import ProblemFolder
import Tools


manage_addProblemSetFolderForm = HTMLFile('dtml/problemSetFolder_manageAddForm', globals())

def manage_addProblemSetFolder(self, id, title='', REQUEST=None, showManage=1):
    '''add a new ProblemSet Folder'''

    id    = str(id)
    pf = ProblemSetFolder(self, id, title)
    self._setObject(id, pf)

    self._getOb(id)._afterAdd(self, REQUEST)

    if REQUEST and showManage:
        return self.manage_main(self, REQUEST)


class ProblemSet(FolderishDBObject):
    '''contains/manages data associated with a ProblemSet'''

    meta_type = 'ProblemSet'
    icon = 'misc_/ZQuest/problemSet'

    useUndo   = 0

    allowedSubObjects = ('ProblemFolder', 'DisplayObjectFolder')

    dbTable = ( DBTable('ProblemSet', [],
                        {'name'          : DBAttribute('name',          'varchar(100)'),
                         'introduction'  : DBAttribute('introduction',  'text'        ),
                         'ispublished'   : DBAttribute('ispublished',   'boolean'     ),
                         'problemsgiven' : DBAttribute('problemsgiven', 'int', 'int'  ),
                         'maxscore'      : DBAttribute('maxscore',      'float', 'float' )})
                + FolderishDBObject.dbTable )

    titleDBRepresentation = 'name'

    manageRoles      = ('ManagerOfProblemSet%(problemsetid)s',) 


    def getIntroduction(self, REQUEST):
        '''getExplanation(self, REQUEST) --> return the explanation parsed for dtml syntax; return string'''

        introduction = self.getAttributes()[0].introduction
        html = DT_HTML.HTML(introduction)
        
        return html(client=self, mapping=REQUEST)


    def getProblemSetId(self):
        '''return the id of the problem set; return int'''
        return self.id



_qry_searchAllProblemSets = '''
<dtml-if "name == '' and introduction == ''">
    SELECT id, name, introduction, ispublished
    FROM ProblemSet
    WHERE id=0
<dtml-else>
  SELECT id, name, introduction, ispublished
  FROM ProblemSet
  WHERE
  
    <dtml-if name>
      name ~* '^&dtml-name;' AND
    </dtml-if>

    <dtml-if introduction>
      introduction ~* '^&dtml-introduction;' AND
    </dtml-if>

    1=1;
</dtml-if>
'''


class ProblemSetFolder(DBObjectFolder, DBObjectSQLContainerSupport):
    '''contains all ProblemSets for one project.'''

    connectionID         = 'dbConnection'
    isSQLMethodContainer = 1
    isRoleSupportRoot    = 1

    useUndo   = 0

    meta_type = 'ProblemSetFolder'  
    icon      = 'misc_/ZQuest/problemSetFolder'
    
    manage_options = ( DBObjectFolder.manage_options
                       + DBObjectSQLContainerSupport.manage_options )

    allowedSubObjects = ('ProblemSet',)

    localManageRoles = ('Instructor',)
    manageRoles      = ('ManagerOfProblemSets',) 
    viewRoles        = ('Student', 'Instructor')

    sqlMethods = ( ('qry_searchAllProblemSets', 'name="" introduction=""', _qry_searchAllProblemSets), )


    def search(self, name='', introduction=''):
        '''search for problem sets; return dbrows;'''

        toRegex = Tools.replaceStarForRegExStar

        container = self._getSQLMethodContainer(self)
        return container.qry_searchAllProblemSets(name=toRegex(name), introduction=toRegex(introduction))
