# This code is copyrighted by Stephan Richter in August, 2000.
#
# written by Stephan Richter
# Last updated: March 18, 2001

from Products.DBObjects import DBObjectFolder, FolderishDBObject, DBTable, DBAttribute 
from Products.DBObjects import DBObjectSQLContainerSupport
from Globals            import HTMLFile


manage_addSessionFolderForm = HTMLFile('dtml/sessionFolder_manageAddForm', globals())

def manage_addSessionFolder(self, id, title='', REQUEST=None, showManage=1):
    '''add a new Session Folder'''

    id = str(id)
    pf = SessionFolder(self, id, title)
    self._setObject(id, pf)

    self._getOb(id)._afterAdd(self, REQUEST)

    if REQUEST and showManage:
        return self.manage_main(self, REQUEST)


class Session(FolderishDBObject):
    '''Session class -  contains/manages data associated with a Session'''

    meta_type = 'Session'
    icon = 'misc_/ZQuest/session'

    useUndo   = 0

    dbTable = DBTable('Session', [],
                      {'personid'          : DBAttribute('personid',          'int', 'int'),
                       'classproblemsetid' : DBAttribute('classproblemsetid', 'int', 'int'),
                       'currentproblemid'  : DBAttribute('currentproblemid',  'int', 'int'),
                       'currentscore'      : DBAttribute('currentscore',      'float', 'float'),
                       'currentmaxscore'   : DBAttribute('currentmaxscore',   'float', 'float'),
                       'startdatetime'     : DBAttribute('startdatetime',     'datetime')}              
                      )

    allowedSubObjects = ('SessionProblemFolder',)

    titleDBRepresentation = None

    manageRoles      = ('ManagerOfSession%(sessionid)s',) 


    def getSessionId(self):
        '''getSessionId(self) --> get the sessions id; return id'''
        return self.id
        

    def getProblemSetAttributes(self):    
        '''getProblemSetAttributes(self) --> get the Problem Set attributes; return dbrow'''

        # Let's get the Problem Set Id first
        classproblemsetid = self.getAttributes()[0].classproblemsetid
        problemsetid = self.Classes.qry_getClassProblemSet(id=classproblemsetid)[0].problemsetid
        # Now get the problem set attributes
        return self.ProblemSets.qry_getProblemSet(id=problemsetid)


    def getNextProblemId(self):
        '''getNextProblemId(self) --> get the next Problem; return int'''

        container = self._getSQLMethodContainer(self)
        # Let's get the current problem id
        currentproblemid = self.getAttributes()[0].currentproblemid

        problemIds = self.SessionProblems.objectIds('SessionProblem')
        problemIds.sort()

        # if there is no currentproblemid, then use the first id is the list
        if not currentproblemid:
            return problemIds[0]

        # check for the case that the current problem id is already the
        # last one
        if problemIds.index(str(currentproblemid)) == len(problemIds)-1:
            return 0

        index = problemIds.index(str(currentproblemid))
        return problemIds[index+1]



_qry_getAllSessionsForPerson = '''
SELECT *
  FROM Session
  WHERE <dtml-sqltest personid type="int">
'''


class SessionFolder(DBObjectFolder, DBObjectSQLContainerSupport):
    '''contains all Sessions for one project.'''

    connectionID         = 'dbConnection'
    isSQLMethodContainer = 1
    isRoleSupportRoot    = 1

    useUndo   = 0

    meta_type = 'SessionFolder'  
    icon      = 'misc_/ZQuest/sessionFolder'
    
    manage_options = ( DBObjectFolder.manage_options
                       + DBObjectSQLContainerSupport.manage_options )

    allowedSubObjects = ('Session',)

    localManageRoles = ('Instructor', 'Student')
    manageRoles      = ('ManagerOfSessions',) 

    sqlMethods = ( ('qry_getAllSessionsForPerson', 'personid=0', _qry_getAllSessionsForPerson), )


    def getAllSessionsOfPerson(self, pid):
        '''getAllSessionsOfPerson(self, personid) --> Get all Sessions for a person; return dbrows'''
        
        container = self._getSQLMethodContainer(self)
        return container.qry_getAllSessionsForPerson(personid=pid)




