# This code is copyrighted by Stephan Richter in August, 2000.
#
# written by Stephan Richter
# Last updated: March 18, 2001

from Products.DBObjects import DBObjectFolder, FolderishDBObject, DBTable, DBAttribute 


class SessionProblem(FolderishDBObject):
    '''SessionProblem class -  contains/manages data associated with a SessionProblem'''

    meta_type = 'SessionProblem'
    icon = 'misc_/ZQuest/problem'

    useUndo   = 0

    dbTable = ( DBTable('SessionProblem', [],
                        {'problemid'    : DBAttribute('problemid',    'int', 'int'),
                         'attemptsleft' : DBAttribute('attemptsleft', 'int', 'int'),
                         'iscompleted'  : DBAttribute('iscompleted',  'bool')})
                + FolderishDBObject.dbTable )

    allowedSubObjects = ('SessionVariableFolder',)

 
    def saveProblemVariables(self, REQUEST):
        '''saveProblemVariables(self, REQUEST) --> saves problem variables in the session problem variables;
           return None'''

        problemId = self.getAttributes()[0].problemid
        problemSetId = self.ProblemSets.qry_getProblem(id=problemId)[0].containerid

        variables = self.ProblemSets[problemSetId].Problems[problemId].getAllVariableValues(REQUEST)

        for variable in variables.keys():
            namespace = REQUEST
            namespace.set('variablename', variable)
            namespace.set('numericvalue', variables[variable])
            namespace.set('sessionproblemid', self.id)
            self.SessionVariables.manage_add(namespace, returnId = 1)

            
    def getProblemVariables(self, REQUEST={}):
        '''getProblemVariables(self) --> return dictionary'''

        variables = REQUEST

        rows = self.qry_getAllSubObjects(table='SessionVariable',
                                         containertype='SessionProblem',
                                         containerid=self.id)

        for row in rows:
            variables[row.variablename] = float(row.numericvalue)

        return variables


    def getSessionProblemId(self):
        ''' '''
        return self.id



class SessionProblemFolder(DBObjectFolder):
    '''SessionProblemFolder class - contains all SessionProblems for one project.'''

    meta_type = 'SessionProblemFolder'  
    icon = 'misc_/ZQuest/problemFolder'

    useUndo   = 0
    
    allowedSubObjects = ('SessionProblem',)


    def setupProblems(self, classProblemSet, REQUEST):
        '''setupProblems(self) '''

        REQUEST.set('sessionid', self.getSessionId())
        REQUEST.set('iscompleted', 0)

        for problem in classProblemSet.getClassProblemsForSession():
            REQUEST.set('problemid', problem.problemid)
            REQUEST.set('attemptsleft', problem.attempts)
            sessionProblemId = self.manage_add(REQUEST, subObjectMetaType='SessionProblem', returnId=1)


    def getSessionProblemWithProblemId(self, problemId):
        '''getSessionProblemWithProblemId(self, problemId) --> get the session problem that has the problem
           id problemId; return SessionProblem'''

        for sessionProblem in self.objectValues():
            if int(sessionProblem.getAttributes()[0].problemid) == int(problemId):
                return sessionProblem

        return None
