# This code is copyrighted by PlanDepot.com, Inc. in August, 2000.
#
# written by Stephan Richter
# Last updated: August 12, 2000

from Globals import HTMLFile
import Globals
from Products.ZSQLMethods.SQL import SQL
from OFS.Folder import Folder
from AccessControl.Role import RoleManager

from ProblemSet import ProblemSetFolder
from Session import SessionFolder
from Class import ClassFolder
from Person import PersonFolder

from Products.ZPyGreSQLDA.DA import Connection

import re, string

manage_addZQuestFolderForm = HTMLFile('zQuestFolder_manageAddForm', globals())


def manage_addZQuestFolder(self, id, title='', connection_string='', REQUEST=None):
    '''manage_addZQuestFolder(self, id, title="", connection_string="", REQUEST=None)
       --> add a new ZQuest Folder; return None or HTML'''

    # make sure the id is a string, otherwise nothing will work
    id=str(id)
    title=str(title)

    # initialize the new ZQuestFolder
    zqf=ZQuestFolder()
    zqf.id=id
    zqf.title=title
    
    # add a database connection to the ZQuestFolder
    zqf._setObject('ZQuestConnection', Connection('ZQuestConnection',
                   'ZQuest Connection', connection_string, 'YES'))

    # add the object to the ZODB
    self._setObject(id, zqf)

    if REQUEST is not None:
        return self.manage_main(self, REQUEST)


class ZQuestFolder(Folder):
    '''ZQuestFolder class - contains all projects for one site.'''

    connectionID = 'ZQuestConnection'
    isSQLMethodContainer = 1

    meta_type = 'ZQuestFolder'

    allowedSubObjects = {'ProblemSetFolder': ProblemSetFolder,
                         'SessionFolder': SessionFolder,
                         'ClassFolder': ClassFolder,
                         'PersonFolder': PersonFolder}
    
    subObjectIds = {'ProblemSetFolder': 'ProblemSets',
                    'SessionFolder': 'Sessions',
                    'ClassFolder': 'Classes',
                    'PersonFolder': 'Persons'}

    # set the permissions required for executing a certain management method
    __ac_permissions__=(
        ('View management screens', ['manage_main']),
        )

    manage_options = (({'label':'Contents',           'action':'manage_main'},
                       {'label':'SQL Methods',        'action':'manage_sql'},
                      )
                      + RoleManager.manage_options)

    manage_main = HTMLFile('zQuestFolder_manageMain', globals())
    manage_sql = HTMLFile('manage_sql', globals())
    manage_addForm = HTMLFile('zQuest_manageAddForm', globals())


    def initSQLMethods(self, connectionID = 'pdConnection'):
        '''initSQLMethods(self) --> Init SQL methods; return None'''

        # Project specific SQL queries
    

    def manage_add(self, REQUEST = None, subObjectMetaType = None, returnId = 0, returnObj = 0):
        '''manage_add(self, REQUEST, returnId = 0, subObjectMetaType = None) --> add a sub-object
           to the ZQuestFolder; return None or new sub-object id'''
        
        if (subObjectMetaType in self.allowedSubObjects.keys() and
            self.allowedSubObjects[subObjectMetaType] != None):

            subObjectClass = self.allowedSubObjects[subObjectMetaType]
            subObjectID = self.subObjectIds[subObjectMetaType]
            subObjectTitle = ''
            subObject = subObjectClass(self, subObjectID, subObjectTitle)

            self._setObject(subObjectID, subObject)

            if returnId:
                return subObjectID

            if returnObj:
                return subObject
        

        # Try to go back to the folder's default (main) view
        if REQUEST is not None:
            return self.manage_main(self, REQUEST)


    def findExistingSubObjects(self, REQUEST=None):
        '''findExistingSubObjects(self) --> Fill the Project with sub-objects; return None'''

        for metaType in self.allowedSubObjects.keys():
            folder = self.manage_add(subObjectMetaType=metaType, returnObj=1)
            if folder:
                self[folder.id].findExistingSubObjects()

        if REQUEST is not None:
            return self.manage_main(self, REQUEST)


    def getSQLMethods(self):
        '''getSQLMethods(self) --> return a list of SQL Methods defined in ProjectFolder; return
           list of objects'''
        list=[]
        type=SQL.meta_type
        for ob in self.__dict__.values():
            if hasattr(ob, 'meta_type') and ob.meta_type==type:
                list.append(ob)
        return list



Globals.default__class_init__(ZQuestFolder)
