# This code is copyrighted by Stephan Richter in August, 2000.
#
# written by Stephan Richter
# Last updated: March 18, 2001

from Products.DBObjects     import DBTable, DBAttribute 
from DisplayObject          import DisplayObject

import PIL
import rexec, cStringIO
import popen2, tempfile
import re, os, string, urllib, sys 
import traceback
from math import *
from CoordinateSystem import *

class ZQuestGraph(DisplayObject):
    '''ZQuestGraph class -  contains/manages data associated with a ZQuestGraph'''

    meta_type = 'ZQuestGraph'
    icon = 'misc_/ZQuest/variable'

    manage_options = DisplayObject.manage_options + ({'label':'View',  'action':'manage_view'},)

    dbTable = DBTable('ZQuestGraph', ['DisplayObject'],
                        {'code' : DBAttribute('code', 'text')})

    fonts = {'Arial 8'  : PIL.ImageFont.load(SOFTWARE_HOME+'/Products/ZQuest/Fonts/100/Arial_8_100.pil'),
             'Arial 10' : PIL.ImageFont.load(SOFTWARE_HOME+'/Products/ZQuest/Fonts/100/Arial_10_100.pil'),
             'Arial 12' : PIL.ImageFont.load(SOFTWARE_HOME+'/Products/ZQuest/Fonts/100/Arial_12_100.pil'),
             'Arial 14' : PIL.ImageFont.load(SOFTWARE_HOME+'/Products/ZQuest/Fonts/100/Arial_14_100.pil'),
             'Arial 16' : PIL.ImageFont.load(SOFTWARE_HOME+'/Products/ZQuest/Fonts/100/Arial_16_100.pil')}
    
    colors = {'red':        (255,   0,   0),
              'yellow':     (255, 255,   0),
              'green':      (  0, 255,   0),
              'cyan':       (  0, 255, 255),
              'blue':       (  0,   0, 255),
              'pink':       (255,   0, 255),
              'white':      (255, 255, 255),
              'black':      (  0,   0,   0),
              'light grey': (224, 224, 224),
              'grey':       (200, 200, 200),
              'dark grey':  ( 96,  96,  96), 
              'cbu red':    (146,  21,  47),
              'navy blue':  (  0,   0, 128)}


    def getHTMLRepresentation(self, REQUEST):
        '''getHTMLRepresentation(REQUEST) --> returns the HTML code representating the object; return string'''

        argList = []

        try:
            variables = self.getPreviousProblemVariables(REQUEST)
        except:
            variables = {}

        if hasattr(self, 'GivenVariables'):
            for variable in self.GivenVariables.objectValues():
                if REQUEST.has_key(variable.title):
                    variables[variable.title] = float(REQUEST[variable.title])

        if hasattr(self, 'SearchedVariables'):
            for variable in self.SearchedVariables.objectValues():
                if REQUEST.has_key(variable.title):
                    variables[variable.title] = float(REQUEST[variable.title])

        if REQUEST.has_key('specialVars'):
            for variable in REQUEST.specialVars:
                if REQUEST.has_key(variable):
                    variables[variable] = REQUEST[variable]

        for key in variables.keys():
            argList.append(key + '=' + urllib.quote(str(REQUEST[key])))

        argString = string.join(argList, '&')

        html = '<img src="%s/manage_view?%s">' %(self.absolute_url(), argString)

        return html


    def _transformVariables(self, vars):
        '''_transformVariables(self, vars) --> transform strings to floats; return list of floats'''

        for var in vars.keys():
            try:
                vars[var] = float(vars[var])
            except:
                pass

        return vars


    def manage_view(self, RESPONSE, REQUEST):
        '''manage_view(self, REQUEST, RESPONSE) --> return image bit stream; '''

        env = rexec.RExec()
        env.ok_builtin_modules = ('PIL', 'Image', 'ImageFont', 'ImageDraw', 'math', 'sys',)

        vars = self._transformVariables(REQUEST.form)
        attrs = self.getAttributes()[0]

        img = PIL.Image.new('RGBA', (int(attrs.width), int(attrs.height)), (255, 255, 255))
        frame = PIL.ImageDraw.Draw(img)
        frame.fonts  = self.fonts
        frame.colors = self.colors
        frame.setink(frame.colors['black'])
        frame.setfont(frame.fonts['Arial 10'])

        # Setup animation support
        animation = Animation(img)
        addFrame = animation.addFrame
        setLoops = animation.setLoops
        setDelay = animation.setDelay
        frame.animation = animation

        try:
            exec string.replace(attrs.code, '\015\012', '\n')

        except Exception:
            frame.text((0, 0), 'Error occured, while trying to draw the picture!')
            frame.text((0, 13), 'Type: %s' %sys.exc_info()[0]) 
            frame.text((0, 26), 'Value: %s' %sys.exc_info()[1])
            traceback.print_stack()
            traceback.print_exc()
            
        RESPONSE.setHeader("Content-type", "image/gif")

        if animation.hasFrames():
            output = animation.make()
        else:
            outFile = cStringIO.StringIO()
            img.save(outFile, 'GIF')
            
            outFile.seek(0)
            output = outFile.read()
            outFile.close()

        del img
        return output



class Animation:
    '''This class is responsible of converting single GIFs into an animated one.'''

    command = '/usr/bin/gifsicle --optimize=2 --color-method diversity --delay=%i --loop=%i %s'

    def __init__(self, image, delay=100, loops=-10):
        '''Initialize the object; return None'''
        self.image  = image
        self.frames = []
        self.delay  = delay
        self.loops  = loops
        

    def setDelay(self, delay=100):
        '''Set a new delay value; return None'''
        self.delay = int(delay)


    def setLoops(self, loops=-10):
        '''Set a new amount of loops; return None'''
        self.loops = int(loops)
        

    def hasFrames(self):
        '''Check whether there are any frames in the animation; return int'''
        return len(self.frames)


    def addFrame(self):
        '''Add a frame to the animation; return None'''
        filename = tempfile.mktemp('.gif')
        file = open(filename, 'w')
        self.image.save(file, 'GIF')
        file.close()
        self.frames.append(filename)


    def make(self):
        '''Generate the animation from the frames; return GIF bit stream'''
        frameFiles = string.join(self.frames, ' ')
        file = popen2.popen2(self.command %(self.delay, self.loops, frameFiles))[0]
        output = file.read()
        return output
    
